/******************************************************************************
 *
 * @brief     This file is part of the TouchGFX 4.8.0 evaluation distribution.
 *
 * @author    Draupner Graphics A/S <http://www.touchgfx.com>
 *
 ******************************************************************************
 *
 * @section Copyright
 *
 * This file is free software and is provided for example purposes. You may
 * use, copy, and modify within the terms and conditions of the license
 * agreement.
 *
 * This is licensed software for evaluation use, any use must strictly comply
 * with the evaluation license agreement provided with delivery of the
 * TouchGFX software.
 *
 * The evaluation license agreement can be seen on www.touchgfx.com
 *
 * @section Disclaimer
 *
 * DISCLAIMER OF WARRANTY/LIMITATION OF REMEDIES: Draupner Graphics A/S has
 * no obligation to support this software. Draupner Graphics A/S is providing
 * the software "AS IS", with no express or implied warranties of any kind,
 * including, but not limited to, any implied warranties of merchantability
 * or fitness for any particular purpose or warranties against infringement
 * of any proprietary rights of a third party.
 *
 * Draupner Graphics A/S can not be held liable for any consequential,
 * incidental, or special damages, or any other relief, or for any claim by
 * any third party, arising from your use of this software.
 *
 *****************************************************************************/
#include <gui/model/Model.hpp>
#include <gui/model/ModelListener.hpp>

#ifndef SIMULATOR
extern "C"
{
#include "stm32469i_eval.h"
}
#include "FreeRTOS.h"
#include "queue.h"
extern xQueueHandle msgQueueUI;
#endif

Model::Model() : modelListener(0), lightState(false), waterTemp(18)
{
}

void Model::tick()
{
#ifndef SIMULATOR
    if (lightState)
    {
        BSP_LED_On(LED1);
    }
    else
    {
        BSP_LED_Off(LED1);
    }

    //get new temperature from message queue (if any)
    if (uxQueueMessagesWaiting(msgQueueUI) > 0)
    {
        int newTemp = 0;
        xQueueReceive(msgQueueUI, &newTemp, 0);
        //inform the view if temperature changed
        if (newTemp != waterTemp)
        {
            waterTemp = newTemp;
            modelListener->temperatureChanged();
        }
    }
#endif
}
